@@ -6,12 +6,14 @@ import os
 import uuid
 from abc import abstractmethod
 from contextlib import contextmanager
+import typing
 
 import mlflow
 import mlflow.utils.autologging_utils
 from mlflow.entities.run_status import RunStatus
 from mlflow.tracking.client import MlflowClient
 from mlflow.utils import gorilla
+from mlflow.utils import is_iterator
 from mlflow.utils.autologging_utils import _logger
 from mlflow.utils.autologging_utils.events import AutologgingEventLogger
 from mlflow.utils.autologging_utils.logging_and_warnings import (
@@ -500,7 +502,14 @@ def safe_patch(
                     def call_original(*og_args, **og_kwargs):
                         def _original_fn(*_og_args, **_og_kwargs):
                             if is_testing():
-                                _validate_args(args, kwargs, og_args, og_kwargs)
+                                _validate_args(
+                                    autologging_integration,
+                                    function_name,
+                                    args,
+                                    kwargs,
+                                    og_args,
+                                    og_kwargs,
+                                )
                                 # By the time `original` is called by the patch implementation, we
                                 # assume that either: 1. the patch implementation has already
                                 # created an MLflow run or 2. the patch code will not create an
@@ -773,8 +782,110 @@ def _validate_autologging_run(autologging_integration, run_id):
     ), "Autologging run with id {} has a non-terminal status '{}'".format(run_id, run.info.status)
 
 
+class ValidationExemptArgument(typing.NamedTuple):
+    """
+    A NamedTuple representing the properties of an argument that is exempt from validation
+
+    autologging_integration: The name of the autologging integration.
+    function_name: The name of the function that is being validated.
+    type_function: A Callable that accepts an object and returns True if the given object matches
+                   the argument type. Returns False otherwise.
+    positional_argument_index: The index of the argument in the function signature.
+    keyword_argument_name: The name of the argument in the function signature.
+    """
+
+    autologging_integration: str
+    function_name: str
+    type_function: typing.Callable
+    positional_argument_index: int = None
+    keyword_argument_name: str = None
+
+    def matches(
+        self,
+        autologging_integration,
+        function_name,
+        value,
+        argument_index=None,
+        argument_name=None,
+    ):
+        """
+        This method checks if the properties provided through the function arguments matches the
+        properties defined in the NamedTuple.
+
+        :param autologging_integration: The name of an autologging integration.
+        :param function_name: The name of the function that is being matched.
+        :param value: The value of the argument.
+        :param argument_index: The index of the argument, if it is passed as a positional
+                                          argument. Otherwise it is None.
+        :param argument_name: The name of the argument, if it is passed as a keyword
+                                      argument. Otherwise it is None.
+        :return: Returns True if the given function properties matches the exempt argument's
+                 properties. Returns False otherwise.
+        """
+        return (
+            self.autologging_integration == autologging_integration
+            and self.function_name == function_name
+            and (
+                self.positional_argument_index == argument_index
+                or self.keyword_argument_name == argument_name
+            )
+            and self.type_function(value)
+        )
+
+
+# WARNING: Exemptions should NOT be introduced unless absolutely necessary. If deemed necessary,
+#          clear reasons must be provided as comment in addition to thorough integration tests.
+_VALIDATION_EXEMPT_ARGUMENTS = [
+    # When extracting implicitly defined `batch_size` in the case that `x` is a generator or a
+    # generator class, we need to consume and restore the first element back to the generator to
+    # calculate the `batch_size`. This means that:
+    # 1. The type of `x` will become 'generator' regardless if user provided `x` as a generator or a
+    #    custom generator class.
+    # 2. The instance of `x` will be different, since we reconstructed the generator after consuming
+    #    the first element.
+    ValidationExemptArgument("tensorflow", "fit", is_iterator, 1, "x")
+]
+
+
+def _is_arg_exempt_from_validation(
+    autologging_integration,
+    function_name,
+    argument,
+    argument_index=None,
+    argument_name=None,
+):
+    """
+    This function is responsible for determining whether or not an argument is exempt from autolog
+    safety validations. This includes both type checking and immutable checking.
+
+    :param autologging_integration: The name of the autologging integration.
+    :param function_name: The name of the function that is being validated.
+    :param argument: The actual argument.
+    :param argument_index: The index of the argument, if it is passed as a positional
+                                      argument. Otherwise it is None.
+    :param argument_name: The name of the argument, if it is passed as a keyword argument.
+                                  Otherwise it is None.
+    :return: True or False
+    """
+    return any(
+        exemption.matches(
+            autologging_integration,
+            function_name,
+            argument,
+            argument_index,
+            argument_name,
+        )
+        for exemption in _VALIDATION_EXEMPT_ARGUMENTS
+    )
+
+
 def _validate_args(
-    user_call_args, user_call_kwargs, autologging_call_args, autologging_call_kwargs
+    autologging_integration,
+    function_name,
+    user_call_args,
+    user_call_kwargs,
+    autologging_call_args,
+    autologging_call_kwargs,
 ):
     """
     Used for testing purposes to verify that, when a patched ML function calls its underlying
@@ -820,6 +931,24 @@ def _validate_args(
                 "such exception safe functions / classes.".format(inp)
             )
 
+    def _assert_autologging_input_positional_args_are_superset(
+        autologging_call_input, user_call_input
+    ):
+        length_difference = len(autologging_call_input) - len(user_call_input)
+        assert (
+            length_difference >= 0
+        ), "{} expected inputs are missing from the call to the original function.".format(
+            length_difference
+        )
+
+    def _assert_autologging_input_kwargs_are_superset(autologging_call_input, user_call_input):
+        assert set(user_call_input.keys()).issubset(set(autologging_call_input.keys())), (
+            "Keyword or dictionary arguments to original function omit"
+            " one or more expected keys: '{}'".format(
+                set(user_call_input.keys()) - set(autologging_call_input.keys())
+            )
+        )
+
     def _validate(autologging_call_input, user_call_input=None):
         """
         Validates that the specified `autologging_call_input` and `user_call_input`
@@ -833,7 +962,11 @@ def _validate_args(
               be equivalent to `user_call_input` or be a superset of `user_call_input`
             - for all other input types, `autologging_call_input` and `user_call_input`
               must be equivalent by reference equality or by object equality
+
+        :param autologging_call_input: call input from autologging
+        :param user_call_input: call input from user
         """
+
         if user_call_input is None and autologging_call_input is not None:
             _validate_new_input(autologging_call_input)
             return
@@ -845,11 +978,8 @@ def _validate_args(
         )
 
         if type(autologging_call_input) in [list, tuple]:
-            length_difference = len(autologging_call_input) - len(user_call_input)
-            assert (
-                length_difference >= 0
-            ), "{} expected inputs are missing from the call" " to the original function.".format(
-                length_difference
+            _assert_autologging_input_positional_args_are_superset(
+                autologging_call_input, user_call_input
             )
             # If the autologging call input is longer than the user call input, we `zip_longest`
             # will pad the user call input with `None` values to ensure that the subsequent calls
@@ -857,12 +987,7 @@ def _validate_args(
             for a, u in itertools.zip_longest(autologging_call_input, user_call_input):
                 _validate(a, u)
         elif type(autologging_call_input) == dict:
-            assert set(user_call_input.keys()).issubset(set(autologging_call_input.keys())), (
-                "Keyword or dictionary arguments to original function omit"
-                " one or more expected keys: '{}'".format(
-                    set(user_call_input.keys()) - set(autologging_call_input.keys())
-                )
-            )
+            _assert_autologging_input_kwargs_are_superset(autologging_call_input, user_call_input)
             for key in autologging_call_input.keys():
                 _validate(autologging_call_input[key], user_call_input.get(key, None))
         else:
@@ -874,8 +999,34 @@ def _validate_args(
                 " Original: '{}'. Expected: '{}'".format(autologging_call_input, user_call_input)
             )
 
-    _validate(autologging_call_args, user_call_args)
-    _validate(autologging_call_kwargs, user_call_kwargs)
+    # Similar validation logic found in _validate, unraveling the list of arguments to exclude
+    # checks for any validation exempt positional arguments.
+    _assert_autologging_input_positional_args_are_superset(autologging_call_args, user_call_args)
+    for index, autologging_call_arg, user_call_arg in itertools.zip_longest(
+        range(len(user_call_args)), autologging_call_args, user_call_args
+    ):
+        if not _is_arg_exempt_from_validation(
+            autologging_integration,
+            function_name,
+            user_call_arg,
+            argument_index=index,
+        ):
+            _validate(autologging_call_arg, user_call_arg)
+
+    # Similar validation logic found in _validate, unraveling the dictionary of arguments to exclude
+    # checks for any validation exempt keyword arguments.
+    _assert_autologging_input_kwargs_are_superset(autologging_call_kwargs, user_call_kwargs)
+    for key in autologging_call_kwargs.keys():
+        if not _is_arg_exempt_from_validation(
+            autologging_integration,
+            function_name,
+            user_call_kwargs.get(key, None),
+            argument_name=key,
+        ):
+            _validate(
+                autologging_call_kwargs[key],
+                user_call_kwargs.get(key, None),
+            )
 
 
 __all__ = [
